"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const utils_1 = require("./utils");
class Activation {
    constructor() {
        this.data = {};
    }
    async execute(ctx, onError) {
        // Allow the arguments to be set outside of any interceptor
        //    For example set them before we call execute. This is so we can use
        //    the activation outside of an environment using activation chains
        if (this.method.parameters.length > 0 && lodash_1.isNil(ctx.getArguments())) {
            ctx.setArguments(new Array(this.method.parameters.length));
        }
        ctx.beforeActivationLength = this.beforeActivation.length;
        let beforeInterceptor = null;
        try {
            let resultValue = false;
            for (ctx.beforeActivationIdx = 0; ctx.beforeActivationIdx < ctx.beforeActivationLength; ++ctx.beforeActivationIdx) {
                beforeInterceptor = this.beforeActivation[ctx.beforeActivationIdx];
                const result = beforeInterceptor.before(ctx);
                resultValue = false;
                if (utils_1.isAwaitable(result)) {
                    resultValue = await result;
                }
                else {
                    resultValue = result;
                }
                // We do not continue with the processing
                if (resultValue === false) {
                    break;
                }
            }
        }
        catch (err) {
            if (lodash_1.isFunction(onError)) {
                onError(beforeInterceptor, err);
            }
            ctx.setError(err);
        }
        let afterActivation = null;
        for (let idx = this.afterActivation.length - 1; idx >= 0; idx--) {
            afterActivation = this.afterActivation[idx];
            // We don't care about after activation result
            try {
                const result = afterActivation.after(ctx);
                if (utils_1.isAwaitable(result)) {
                    await result;
                }
            }
            catch (err) {
                if (lodash_1.isFunction(onError)) {
                    onError(beforeInterceptor, err);
                }
                ctx.setError(err);
            }
        }
        return ctx;
    }
    removeBeforeActivation(activation, context) {
        const idx = this.beforeActivation.indexOf(activation);
        if (idx >= 0) {
            this.beforeActivation.splice(idx, 1);
            context.beforeActivationIdx--;
            context.beforeActivationLength--;
        }
    }
    removeAfterActivation(activation, context) {
        const idx = this.afterActivation.indexOf(activation);
        if (idx >= 0) {
            this.afterActivation.splice(idx, 1);
        }
    }
}
exports.Activation = Activation;
//# sourceMappingURL=activation.execution.js.map